/********************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE TVPAINT ANIMATION LOADER/SAVER PLUGINS  *
 * SOURCE CODE.                                                     *
 *                                                                  *
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS LIBRARY SOURCE IS     *
 * GOVERNED BY A FREEBSD SOURCE LICENSE INCLUDED WITH THIS SOURCE   *
 * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
 *                                                                  *
 * THE TVPAINT ANIMATION LOADER/SAVER PLUGINS SOURCE CODE IS (C)    *
 * COPYRIGHT 2010-2015 BY THE TVPAINT DEVELOPPEMENT COMPANY         *
 * http://www.tvpaint.com/                                          *
 *                                                                  *
 ********************************************************************/

// ==================================================================
/** @file Format.Base.h
      @brief Header File: Format.Base
     @author Michael Schreiner & Herve ADAM
      $Date: $
        $Id: $
  @copyright (c) 2010-2015 TVPaint Developpement. All Rights Reserved.
*/
// ==================================================================

#ifndef __FORMAT_BASE_HH
#define __FORMAT_BASE_HH

#include "Version.h"

#include <stdint.h>

/**
* Fields order
*/
enum  eFieldOrder
{
    kNone       = 0,
    kLowerFirst = 1,
    kUpperFirst = 2
};

/**
* Type of output sound channels
*/
enum  eChannelTypes
{
    kChannelTypesUnknown = 0,
    kChannelTypesMono,
    kChannelTypesStereo
};

/**
* One BGRA Pixel:
*   Blue
*   Green
*   Red
*   Alpha
*/

struct cPixel32
{
public:
    cPixel32():
        mBlue(0),
        mGreen(0),
        mRed(0),
        mAlpha(0)
    {
    }
    cPixel32(uint8_t iBlue, uint8_t iGreen, uint8_t iRed, uint8_t iAlpha ):
        mBlue(iBlue),
        mGreen(iGreen),
        mRed(iRed),
        mAlpha(iAlpha)
    {
    }
    cPixel32(const cPixel32& iSrc ):
        mBlue( iSrc.mBlue ),
        mGreen( iSrc.mGreen ),
        mRed( iSrc.mRed ),
        mAlpha(  iSrc.mAlpha )
    {
    }

public:
    /** Blue getter & setter */
    uint8_t Blue() { return mBlue; }
    const uint8_t Blue() const { return mBlue; }
    void Blue(uint8_t iBlue) { mBlue = iBlue; }
    /** Green getter & setter */
    const uint8_t Green() const { return mGreen; }
    uint8_t Green() { return mGreen; }
    void Green(uint8_t iGreen) { mGreen = iGreen; }
    /** Red getter & setter */
    const uint8_t Red() const { return mRed; }
    uint8_t Red() { return mRed; }
    void Red(uint8_t iRed) { mRed = iRed; }
    /** Alpha getter & setter */
    const uint8_t Alpha() const { return mAlpha; }
    uint8_t Alpha() { return mAlpha; }
    void Alpha(uint8_t iAlpha) { mAlpha = iAlpha; }

public:
    uint8_t mBlue;
    uint8_t mGreen;
    uint8_t mRed;
    uint8_t mAlpha;
};


/**
* One Block of Pixel (width * height)
*/
struct cPixelBlockFF
{
public:
    cPixelBlockFF(int iWidth, int iHeight, cPixel32* iPixels ):
        mWidth(iWidth),
        mHeight(iHeight),
        mPixels(iPixels)
    {
    }

public:
    /** Width getter & setter */
    int  Width() const { return  mWidth; }
    void Width( int iWidth ) { mWidth = iWidth; }
    /** Height getter & setter */
	int  Height() const { return  mHeight; }
	void Height( int iHeight ) { mHeight = iHeight; }
    /** Pixels getter & setter */
    const cPixel32* Pixels() const { return mPixels;}
    cPixel32* Pixels() { return mPixels;}
    void Pixels(cPixel32* iPixels) { mPixels = iPixels; }
    /** Pixels Row(X, Y) getter */
    const cPixel32* Pixels(int iX, int iY) const { return &mPixels[iX + iY * mWidth]; }
    cPixel32* Pixels(int iX, int iY) { return &mPixels[iX + iY * mWidth]; }

private:
	int mWidth;
    int mHeight;
    cPixel32* mPixels;
};


/**
* One Block of Samples to write
*/
class cAudioBlock
{
public:
    virtual  ~cAudioBlock() {}

public:
	virtual  void  WriteSamples( const void* iSamples, int iSampleCount ) = 0; //iSampleCount in Sample unit not in bytes unit

};


#endif // __FORMAT_BASE_HH


