/********************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE TVPAINT ANIMATION LOADER/SAVER PLUGINS  *
 * SOURCE CODE.  													*
 *																	*
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS LIBRARY SOURCE IS     *
 * GOVERNED BY A FREEBSD SOURCE LICENSE INCLUDED WITH THIS SOURCE   *
 * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
 *                                                                  *
 * THE TVPAINT ANIMATION LOADER/SAVER PLUGINS SOURCE CODE IS (C)    *
 * COPYRIGHT 2010-2015 BY THE TVPAINT DEVELOPPEMENT COMPANY         *
 * http://www.tvpaint.com/                                          *
 *                                                                  *
 ********************************************************************/

// ==================================================================
/** @file Format.BaseSaver.h
      @brief Header File: Format.BaseSaver
     @author Michael Schreiner & Herve ADAM
      $Date: $
        $Id: $
  @copyright (c) 2010-2015 TVPaint Developpement. All Rights Reserved.
*/
// ==================================================================

#ifndef __FORMAT_BASESAVER_HH
#define __FORMAT_BASESAVER_HH

#include <stdint.h>

#ifndef __FORMAT_BASE_HH
#include "Format.Base.h"
#endif

#define MAX_USABLE_FORMAT 25

/**
* Soundtrack of the movie
*/
class cSaverAudioSequence
{
public:
    virtual ~cSaverAudioSequence()
    {
    }

    cSaverAudioSequence():
        mChannelCount(0),
        mChannelType(kChannelTypesUnknown),
        mBitsPerChannel(0),
        mBytesPerSample(0),
        mOffset(0),
        mSampleCount(0),
        mSamplingRate(0)
    {
    }
    cSaverAudioSequence(int iChannelCount, eChannelTypes iChannelType, int iBitsPerChannel, int iBytesPerSample, double iOffset, int iSampleCount, int iSamplingRate):
        mChannelCount(iChannelCount),
        mChannelType(iChannelType),
        mBitsPerChannel(iBitsPerChannel),
        mBytesPerSample(iBytesPerSample),
        mOffset(iOffset),
        mSampleCount(iSampleCount),
        mSamplingRate(iSamplingRate)
    {
    }

public:
    /** Channel count getter & setter */
    int  ChannelCount() { return mChannelCount; }
    void ChannelCount( int iChannelCount ) { mChannelCount = iChannelCount; }
    /** Channel type getter & setter */
    eChannelTypes ChannelType() { return mChannelType; }
    void ChannelType( eChannelTypes iChannelType ) { mChannelType = iChannelType; }
    /** Bit per channel getter & setter */
    int  BitsPerChannel() { return mBitsPerChannel; }
    void BitsPerChannel( int iBitsPerChannel ) { mBitsPerChannel = iBitsPerChannel; }
    /** Bytes per sample getter & setter */
    int  BytesPerSample() { return mBytesPerSample / mChannelCount; }
    void BytesPerSample( int iBytesPerSample ) { mBytesPerSample = iBytesPerSample; }
    /** Start offset in seconds getter & setter */
    double Offset() { return mOffset; }
    void   Offset( double iOffset ) { mOffset = iOffset; }
    /** Sample count getter & setter */
    int  SampleCount() { return mSampleCount; }
    void SampleCount( int iSampleCount ) { mSampleCount = iSampleCount; }
    /** Sampling rate in Hz getter & setter */
    int  SamplingRate() { return mSamplingRate; }
    void SamplingRate( int iSamplingRate ) { mSamplingRate = iSamplingRate; }

public:
    /** Read iSampleCount from iStartSample */
    virtual int  ReadData( int16_t* oBuffer, int iSampleCount, int* oSampleCount, int iStartSample ) = 0;

protected:
    int mChannelCount;
    eChannelTypes mChannelType;
    int mBitsPerChannel;
    int mBytesPerSample;    // 16bits stereo(*2) => 4 bytes
    double mOffset;
    int mSampleCount;
    int mSamplingRate;


};


/**
* Videotrack of the movie
*/

class cSaverFrameSequence
{
public:
    virtual ~cSaverFrameSequence()
    {
    }

    cSaverFrameSequence():
        mWidth(0),
        mHeight(0),
        mFieldOrder(kNone),
        mFrameCount(0),
        mFrameRate(0),
        mPixelAspectRatio(0),
        mProgress(0),
        mBreakSignal(0)
    {
    }

    cSaverFrameSequence(int iWidth, int iHeight, eFieldOrder iFieldOrder, int iFrameCount, double iFrameRate, double iPixelAspectRatio):
        mWidth(iWidth),
        mHeight(iHeight),
        mFieldOrder(iFieldOrder),
        mFrameCount(iFrameCount),
        mFrameRate(iFrameRate),
        mPixelAspectRatio(iPixelAspectRatio),
        mProgress(0),
        mBreakSignal(0)
    {
    }

public:
    /** Width getter & setter */
    int  Width() const { return  mWidth; }
    void Width( int iWidth ) { mWidth = iWidth; }
    /** Height getter & setter */
	int  Height() const { return  mHeight; }
	void Height( int iHeight ) { mHeight = iHeight; }
	/** Field order getter & setter */
	eFieldOrder FieldOrder() const { return  mFieldOrder; }
	void FieldOrder( eFieldOrder iFieldOrder ) { mFieldOrder = iFieldOrder; }
	/** Frame count getter & setter */
	int  FrameCount() const { return  mFrameCount; }
	void FrameCount( int64_t iFrameCount ) { mFrameCount = iFrameCount; }
	/** Frame rate getter & setter */
	double FrameRate() const { return  mFrameRate; }
	void   FrameRate(double iFrameRate ) { mFrameRate = iFrameRate; }
	/** Pixel aspect ratio getter & setter */
	double PixelAspectRatio() const { return  mPixelAspectRatio; }
	void   PixelAspectRatio( double iPixelAspectRatio ) { mPixelAspectRatio = iPixelAspectRatio; }

	bool BreakSignal() const { return mBreakSignal; }

    void Progress(int iProgress) { mProgress = iProgress; }

public:
	int     		mWidth;
    int     		mHeight;
	eFieldOrder  	mFieldOrder;
	int     		mFrameCount;
	double  		mFrameRate;
	double  		mPixelAspectRatio;
	int     		mProgress;
	bool    		mBreakSignal;

public:
    /** Read a frame at iFramePosition in oDst */
    virtual int ReadFrame( int iFramePosition, cPixelBlockFF* oDst ) = 0;
};

//SaveMode
enum  eSaverMode
{
    bSavePalette = (1<<0),
    bSaveRGB	 = (1<<1),
    bSaveRGBA	 = (1<<2),
    bSaveAlpha   = (1<<3),	//Alpha Only
    bSaveGrey	 = (1<<4),

    bBlendMask   = (1<<5), // option save Alpha
    bSaveYUV	 = (1<<6),
    bSaveMJPG	 = (1<<7),
};

/**
* Output movie format
*/
class cSaverFormat
{
public:
    virtual  ~cSaverFormat()
    {
    }

    /** Format copyrights */
    virtual const char* Copyright() = 0;
    /** Output file extension */
    virtual const char* Extension() = 0;
    /** Format name displayed in TVP */
    virtual const char* FormatName() = 0;
    /** Output format presets */
    virtual const char* Options( int iIdx ) = 0;
    /** Select Output format presets */
    virtual void Option( eSaverMode iIdx ) = 0;
	/** The version */
	virtual const char* Version() = 0;
    /**
     * Save movie (iFrameSequence and iAudioSequence if not null) in file iFilePath
     * with the preset iOptionIdx and the bitrate iBitRate if it is use to encode file.
     * If output file doesn't exist, it will be create
     *
     * iOptionIdx : -1 is not a valid option
     */
    virtual int Save(const char* iFilePath, cSaverFrameSequence* iFrameSequence, cSaverAudioSequence* iAudioSequence, int iOptionIdx, int iBitRate) = 0;
    /** Output bitrate useful */
    virtual bool UseBitrate() = 0;

    virtual bool Audio() = 0;

    virtual uint8_t SaverMode() = 0;
    
    virtual const char* Error() { return 0; }

};

/** The types of the class factories */
typedef cSaverFormat* createSaverFormat_t();


#endif // __FORMAT_BASE_HH
