#!/bin/bash

#--- for x86 build

curl
if [ $? -eq 127 ]; then	
	echo "### INSTALL CURL ###"
	exit 10
fi

YASM_PATH=`pwd`/additional-stuff/yasm/_install/bin
if [ ! -d "$YASM_PATH" ]; then
	(
	mkdir -p additional-stuff
	cd additional-stuff
	curl -O http://www.tortall.net/projects/yasm/releases/yasm-1.3.0.tar.gz
	tar xvf yasm*
	find . -maxdepth 1 -type d -name "yasm*" -exec mv {} yasm \;
	cd yasm
	./configure --prefix=`pwd`/_install
	make
	make install
	)
fi

export PATH=$PATH:$YASM_PATH

#---

usage()
{
	nom=`basename $0`
	echo "*****"
	echo "	$nom -s (10.6|10.7|...) -a (32|64) -c (debug|release) -o /my/absolute/install/path"
	echo "	$nom -h"
	echo "*****"
}

sdk=""
arch=0
configuration=""
outpath=""

# Cas particulier de l'option -h: doit être seule
if [[ $# -eq 0 ]]; then usage; exit 2; fi
if [[ $# -eq 1 && "$1" = "-h" ]]; then usage; exit 2; fi

#http://www.shellunix.com/ksh.html
while getopts ":s:a:c:o:" option
do
	case $option in
		s) sdk=$OPTARG
		;;
		:) usage
			exit 1
		;;

		a) arch=$OPTARG
		;;
		:) usage
			exit 1
		;;

		c) configuration=$OPTARG
		;;
		:) usage
			exit 1
		;;

		o) outpath=$OPTARG
		;;
		:) usage
			exit 1
		;;
		
		\?) usage
			exit 1
		;;
	esac
done

if [[ x"$sdk" = x ]]; then
	usage
	exit 1
fi
if [[ $arch -ne 32 && $arch -ne 64 ]]; then
	usage
	exit 1
fi
if [[ x"$configuration" != x"debug" && x"$configuration" != x"release" ]]; then
	usage
	exit 1
fi
if [[ x"$outpath" = x ]]; then
	usage
	exit 1
fi

#---

arch_flag=-m$arch
debug_flag=--disable-debug
config=
if [[ x"$configuration" = x"debug" ]]; then
	debug_flag=--enable-debug
	config=d
fi

zlib_flags=
zlib_flags="${zlib_flags} --extra-cflags=-I$outpath/include"
zlib_flags="${zlib_flags} --extra-ldflags=-L$outpath/lib"
zlib_flags="${zlib_flags} --extra-ldflags=-lz"

intdir=macosx$arch$config-$sdk-lgpl

#---

rm -rf $intdir

(
mkdir $intdir
cd $intdir

export MACOSX_DEPLOYMENT_TARGET=$sdk
CFLAGS="$arch_flag"  CXXFLAGS="$arch_flag" LDFLAGS="$arch_flag" PKG_CONFIG_PATH=${outpath}/lib/pkgconfig/ ../../../configure  --prefix=$outpath $zlib_flags $debug_flag \
																--enable-pic --enable-libopenh264 --enable-libvpx --enable-libopus --disable-vaapi --disable-vdpau --disable-nvenc \
																--disable-nonfree --disable-ffmpeg --disable-ffplay --disable-ffserver --disable-ffprobe --disable-doc \
																--enable-static --disable-shared --disable-gnutls --disable-protocol=tls --disable-openssl \
																--disable-lzma --disable-bzlib --disable-xlib --disable-iconv \
																--disable-libxcb --disable-libxcb-shm --disable-libxcb-xfixes --disable-libxcb-shape
make -j12
make install
)

